# coding: utf-8
"frozen and hidden values"
import autopath
from py.test import raises

from tiramisu.config import Config
from tiramisu.option import ChoiceOption, BoolOption, IntOption, FloatOption, \
    StrOption, OptionDescription
from tiramisu.error import PropertiesOptionError


def make_description():
    gcoption = ChoiceOption('name', 'GC name', ('ref', 'framework'), 'ref')
    gcdummy = BoolOption('dummy', 'dummy', default=False, properties=(('hidden'),))
    objspaceoption = ChoiceOption('objspace', 'Object space',
                                  ('std', 'thunk'), ['std'], multi=True)
    booloption = BoolOption('bool', 'Test boolean option', default=True)
    intoption = IntOption('int', 'Test int option', default=0)
    floatoption = FloatOption('float', 'Test float option', default=2.3)
    stroption = StrOption('str', 'Test string option', default="abc")

    wantref_option = BoolOption('wantref', 'Test requires', default=False,
                                requires=(('gc.name', 'ref', 'hidden'),))
    wantframework_option = BoolOption('wantframework', 'Test requires',
                                      default=False,
                                      requires=(('gc.name', 'framework', 'hidden'),))

    # ____________________________________________________________
    booloptiontwo = BoolOption('booltwo', 'Test boolean option two', default=False)
    subgroup = OptionDescription('subgroup', '', [booloptiontwo])
    # ____________________________________________________________

    gcgroup = OptionDescription('gc', '', [subgroup, gcoption, gcdummy, floatoption])
    descr = OptionDescription('trs', '', [gcgroup, booloption, objspaceoption,
                                          wantref_option, stroption,
                                          wantframework_option,
                                          intoption])
    return descr


# ____________________________________________________________
def test_is_hidden():
    descr = make_description()
    config = Config(descr)
    setting = config.cfgimpl_get_settings()
    setting.read_write()
    dummy = config.unwrap_from_path('gc.dummy')
    assert not config.cfgimpl_get_settings().has_property('frozen', dummy)
    # setattr
    raises(PropertiesOptionError, "config.gc.dummy == False")
    # getattr
    raises(PropertiesOptionError, "config.gc.dummy")


def test_group_is_hidden():
    descr = make_description()
    config = Config(descr)
    setting = config.cfgimpl_get_settings()
    setting.read_write()
    gc = config.unwrap_from_path('gc')
    config.unwrap_from_path('gc.dummy')
    config.cfgimpl_get_settings().add_property('hidden', gc)
    raises(PropertiesOptionError, "config.gc.dummy")
    assert config.cfgimpl_get_settings().has_property('hidden', gc)
    raises(PropertiesOptionError, "config.gc.float")
    # manually set the subconfigs to "show"
    config.cfgimpl_get_settings().del_property('hidden', gc)
    assert not config.cfgimpl_get_settings().has_property('hidden', gc)
    assert config.gc.float == 2.3
    #dummy est en hide
    prop = []
    try:
        config.gc.dummy = False
    except PropertiesOptionError, err:
        prop = err.proptype
    assert 'hidden' in prop


def test_group_is_hidden_multi():
    descr = make_description()
    config = Config(descr)
    setting = config.cfgimpl_get_settings()
    setting.read_write()
    obj = config.unwrap_from_path('objspace')
    objspace = config.objspace
    config.cfgimpl_get_settings().add_property('hidden', obj)
    raises(PropertiesOptionError, "config.objspace")
    assert config.cfgimpl_get_settings().has_property('hidden', obj)
    prop = []
    try:
        objspace.append('std')
    except PropertiesOptionError, err:
        prop = err.proptype
    assert 'hidden' in prop
    config.cfgimpl_get_settings().del_property('hidden', obj)
    assert not config.cfgimpl_get_settings().has_property('hidden', obj)
    config.objspace.append('std')


def test_global_show():
    descr = make_description()
    config = Config(descr)
    setting = config.cfgimpl_get_settings()
    setting.read_write()
    dummy = config.unwrap_from_path('gc.dummy')
    config.cfgimpl_get_settings().add_property('hidden', dummy)
    assert config.cfgimpl_get_settings().has_property('hidden', dummy)
    raises(PropertiesOptionError, "config.gc.dummy == False")


def test_with_many_subgroups():
    descr = make_description()
    config = Config(descr)
    booltwo = config.unwrap_from_path('gc.subgroup.booltwo')
    assert not config.cfgimpl_get_settings().has_property('hidden', booltwo)
    assert config.gc.subgroup.booltwo is False
    config.cfgimpl_get_settings().add_property('hidden', booltwo)
    path = 'gc.subgroup.booltwo'
    homeconfig, name = config.cfgimpl_get_home_by_path(path)
    assert name == "booltwo"
    getattr(homeconfig._cfgimpl_descr, name)
    assert config.cfgimpl_get_settings().has_property('hidden', booltwo)
